var log      = require('npmlog');
var tomdoc   = require('tomdoc/lib/parser');

var parser = module.exports;

// TODO: match with also \t
var matchers = parser.matchers = {};
matchers.class = /^\s*class\s*(\w+)/;
matchers.getter = /^\s*get\s*(\w+)\s*\(\)/;
matchers.constructor = /^\s*(constructor)\s*\(/;

parser.comments = (lines) => {
  var comments = [];
  var buffer = [];

  lines.forEach((line, i) => {
    var next = lines[i + 1];

    if (!/^\s*\/\//.test(line)) return;

    // next line is a comment, fill up the buffer
    if (next && /^\s*\/\//.test(next)) {
      log.silly('parser  ', 'fill up buffer: ', line);
      return buffer.push(line.replace(/^\s*\/\//, ''));
    }

    // next line is not a comment, meaning its potentially a tomdoc block
    buffer.push(line.replace(/^\s*\/\//, ''));

    var identifier = parser.identifier(next);
    comments.push({
      next: next,
      line: line,
      block: buffer.concat(),
      identifier: identifier || next.trim().replace(/\s*{$/, '')
    });

    buffer = [];
  });

  return comments;
}

parser.identifiers = (lines) => {
  return lines.map(parser.identifier).filter((id) => id )
}

parser.identifier = (line) => {
  return Object.keys(matchers).map(parser.matchIdentifier.bind(parser, line))[0];
}

parser.matchIdentifier = (line, matcher) => {
  var matched = parser.match(matchers[matcher], line);
  if (!matched) return '';
  log.info('matched ', line)
  log.info('matched ', 'Line matched as %s: ', matcher, matched);
  return matched;
}

parser.match = (reg, line) => {
  var m = line.match(reg);
  return m && m[1];
}

parser.tomdoc = (block, privates) => {
  var tom = tomdoc.parse(block);
  if (privates) return tom;
  if (tom.status === 'Private') return;
  return tom;
}

parser.tomdocs = (comments) => {
  return comments.map((comment) => {
    var doc = parser.tomdoc(comment.block.join('\n'));
    if (!doc) return;
    doc.identifier = comment.identifier;
    if (doc.examples) doc.examples = doc.examples.join('\n');
    return doc;
  }).filter((doc) => {
    return doc;
  })
}
