var fs     = require('fs');
var path   = require('path');
var mkdirp = require('mkdirp');
var hbs    = require('handlebars').compile;
var log    = require('npmlog');
var exists = fs.existsSync;
var read   = fs.readFileSync;
var write  = fs.writeFileSync;

var index = hbs(read(path.join(__dirname, '../templates/mdl/index.html'), 'utf8'));
var file = hbs(read(path.join(__dirname, '../templates/mdl/file.html'), 'utf8'));

var template = module.exports;

template.generate = (data, opts) => {
  log.info('template', 'Generate documentation with options:', opts);
  if (opts.json) return console.log(JSON.stringify(data, null, 2));
  if (opts.template) {
    index = hbs(read(path.join(opts.template, 'index.html'), 'utf8'));
    file = hbs(read(path.join(opts.template, 'file.html'), 'utf8'));
  }

  var pkgjson = exists('package.json') ? require(path.resolve('package.json')) : {};

  data.forEach((page) => {
    var content = (page.file === 'index.html' ? index : file)(Object.assign({}, page, opts, pkgjson, {
      title: pkgjson.name || path.basename(process.cwd())
    }));

    var filename = template.filename(page.filename, page);
    var dest = path.join(opts.output || 'docs', filename, 'index.html');

    log.info('template', 'Generate %s documentation at %s', page.filename, dest);
    log.silly('template', content);

    if (!(opts.noop || opts.debug)) {
      mkdirp.sync(path.dirname(dest));
      write(dest, content);
    }
  });
};

template.filename = (filename, page) => {
  filename = filename.replace(path.extname(filename), '');
  log.info('test', filename);
  if (path.basename(filename) === 'index') filename = path.dirname(filename);
  log.info('test', filename);
  log.silly('test', page);
  return filename;
}
